@@ -143,3 +143,8 @@ class Rect(NamedTuple):
         if x1 != x2 and y1 != y2:
             return Rect(x1, y1, x2 - x1, y2 - y1)
         return None
+
+    def normalized_diagonal(self):
+        # used for computing percentages of lengths relative to the SVG viewport:
+        # https://www.w3.org/TR/SVG2/coords.html#Units
+        return math.hypot(self.w, self.h) / math.sqrt(2)
