@@ -114,7 +114,6 @@ def _do_pathop(
         sk_path = pathops.op(sk_path, sk_path2, op, fix_winding=True)
     else:
         sk_path.simplify(fix_winding=True)
-    assert sk_path.fillType == pathops.FillType.WINDING
     return svg_commands(sk_path)
 
 
@@ -184,7 +183,13 @@ def stroke(
     # nuke any conics that snuck in (e.g. with stroke-linecap="round")
     sk_path.convertConicsToQuads(tolerance)
 
-    assert sk_path.fillType == pathops.FillType.WINDING
+    backup = pathops.Path(sk_path)
+    try:
+        sk_path.simplify(fix_winding=True)
+    except pathops.PathOpsError:
+        # skip tricky paths that trigger PathOpsError
+        # https://github.com/googlefonts/picosvg/issues/192
+        sk_path = backup
     return svg_commands(sk_path)
 
 
