@@ -18,6 +18,25 @@ from picosvg.svg_reuse import normalize, affine_between
 import pytest
 
 
+@pytest.mark.parametrize(
+    "shape, tolerance, expected_normalization",
+    [
+        # Real example from Noto Emoji, eyes that were normalizing everything to have 0,0 coords
+        # Caused by initial M and L having same coords, creating a 0-magnitude vec
+        (
+            SVGPath(
+                d="M44.67,45.94 L44.67,45.94 C40.48,45.94 36.67,49.48 36.67,55.36 C36.67,61.24 40.48,64.77 44.67,64.77 L44.67,64.77 C48.86,64.77 52.67,61.23 52.67,55.36 C52.67,49.49 48.86,45.94 44.67,45.94 Z"
+            ),
+            0.1,
+            "M0,0 l0,0 c0.2,-0.3 0.6,-0.3 1,0 c0.4,0.3 0.4,0.7 0.2,1 l0,0 c-0.2,0.3 -0.6,0.3 -1,0 c-0.4,-0.3 -0.4,-0.7 -0.2,-1 z",
+        ),
+    ],
+)
+def test_svg_normalization(shape, tolerance, expected_normalization):
+    normalized = normalize(shape, tolerance)
+    assert normalized.round_floats(4).d == expected_normalization
+
+
 @pytest.mark.parametrize(
     "s1, s2, expected_affine, tolerance",
     [
@@ -44,6 +63,13 @@ import pytest
             Affine2D.identity().translate(1, -1),
             0.01,
         ),
+        # different rects
+        (
+            SVGRect(x=20, y=20, width=100, height=20),
+            SVGRect(x=40, y=30, width=60, height=20),
+            Affine2D(a=0.6, b=0.0, c=0.0, d=1.0, e=28.0, f=10.0),
+            0.01,
+        ),
         # circles that may happen to match the ones Noto clock emoji
         (
             SVGCircle(cx=15.89, cy=64.13, r=4),
@@ -104,6 +130,17 @@ import pytest
             Affine2D(a=0.249, b=-0.859, c=0.859, d=0.249, e=32.255, f=97.667),
             0.2,
         ),
+        # Real example from Noto Emoji, eyes that were normalizing everything to have 0,0 coords
+        (
+            SVGPath(
+                d="M44.67,45.94L44.67,45.94 c-4.19,0-8,3.54-8,9.42 s3.81,9.41,8,9.41l0,0 c4.19,0,8-3.54,8-9.41 S48.86,45.94,44.67,45.94z"
+            ),
+            SVGPath(
+                d="M83,45.94   L83,45.94    c-4.19,0-8,3.54-8,9.42 s3.81,9.41,8,9.41l0,0 c4.19,0,8-3.54,8-9.41 S87.21,45.94,83,45.94z"
+            ),
+            Affine2D.identity().translate(38.33, 0.0),
+            0.1,
+        ),
     ],
 )
 def test_svg_reuse(s1, s2, expected_affine, tolerance):
@@ -117,9 +154,9 @@ def test_svg_reuse(s1, s2, expected_affine, tolerance):
     if expected_affine:
         assert (
             affine
-        ), f"No affine found between {s1.d} and {s2.d}. Expected {expected_affine}"
+        ), f"No affine found between {s1.as_path().d} and {s2.as_path().d}. Expected {expected_affine}"
         # Round because we've seen issues with different test environments when overly fine
         affine = affine.round(4)
     assert (
         affine == expected_affine
-    ), f"Unexpected affine found between {s1.d} and {s2.d}."
+    ), f"Unexpected affine found between {s1.as_path().d} and {s2.as_path().d}."
